﻿Imports System.Data.Common

Public Class MyDbConnector

    Enum Providers
        Odbc = 0
        OleDb = 1
        SqlServer = 2
        SqlServerCe = 3
        Oracle = 4
    End Enum

    Private Function GetProviderName(ByVal Provider As Providers) As String
        Select Case Provider
            Case Providers.Odbc
                Return "System.Data.Odbc"
            Case Providers.Oledb
                Return "System.Data.OleDb"
            Case Providers.SqlServer
                Return "System.Data.SqlClient"
            Case Providers.SqlServerCe
                Return "System.Data.SqlServerCe.3.5"
            Case Providers.Oracle
                Return "System.Data.OracleClient"
        End Select
    End Function

    Dim Fac As DbProviderFactory

    Private ConnectionStr As String

    Sub New(ByVal ConStr As String, ByVal Provider As Providers)
        ConnectionStr = ConStr
        Fac = DbProviderFactories.GetFactory(GetProviderName(Provider))
    End Sub

    Function ExcuteCommand(ByVal CommandText As String, ByVal Parameters(,) As String) As Boolean
        Dim Cn As DbConnection
        Try
            ' إنشاء اتصال
            Cn = Fac.CreateConnection()
            Cn.ConnectionString = ConnectionStr

            Cn.Open()

            Dim Cmd = CreateCommand(CommandText, Parameters, Cn)
            ' تنفيذ الأمر
            Cmd.ExecuteNonQuery()
            ExcuteCommand = True

        Catch ex As Exception
            MsgBox(ex.Message)
            ExcuteCommand = False
        Finally
            Cn.Close()
        End Try
    End Function

    Function GetReader(ByVal CommandText As String, ByVal Parameters(,) As String,
                       Optional ByVal Sequential As Boolean = False) As DbDataReader
        Dim Cn As DbConnection
        Try
            ' إنشاء اتصال
            Cn = Fac.CreateConnection()
            Cn.ConnectionString = ConnectionStr
            Cn.Open()

            Dim Cmd = CreateCommand(CommandText, Parameters, Cn)
            ' تنفيذ الأمر والحصول على قارئ بيانات
            ' لاحظ أننا سنترك الاتصال مفتوحا حتى يمكن استخدام قارئ البيانات
            ' لهذا يجب أن نطلب من قارئ البيانات إغلاق الاتصال عند إغلاقه
            If Sequential Then
                GetReader = Cmd.ExecuteReader(CommandBehavior.SequentialAccess Or CommandBehavior.CloseConnection)
            Else
                GetReader = Cmd.ExecuteReader(CommandBehavior.CloseConnection)
            End If

        Catch ex As Exception
            MsgBox(ex.Message)
            GetReader = Nothing
        End Try
    End Function

    Function GetValue(ByVal CommandText As String, ByVal Parameters(,) As String) As Object
        Dim Cn As DbConnection
        Try
            ' إنشاء اتصال
            Cn = Fac.CreateConnection()
            Cn.ConnectionString = ConnectionStr
            Cn.Open()

            Dim Cmd = CreateCommand(CommandText, Parameters, Cn)
            ' تنفيذ الأمر والحصول على أول قيمة في الناتج
            GetValue = Cmd.ExecuteScalar

        Catch ex As Exception
            MsgBox(ex.Message)
            GetValue = Nothing
        Finally
            If Cn IsNot Nothing Then Cn.Close()
        End Try
    End Function

    Function GetTable(ByVal CommandText As String, Optional ByVal Parameters(,) As String = Nothing) As DataTable
        Dim Cn As DbConnection
        Try
            ' إنشاء اتصال
            Cn = Fac.CreateConnection()
            Cn.ConnectionString = ConnectionStr
            Cn.Open()

            Dim Cmd = CreateCommand(CommandText, Parameters, Cn)

            Dim Table As New DataTable
            Dim Da = Fac.CreateDataAdapter
            Da.SelectCommand = Cmd
            Da.Fill(Table)

            GetTable = Table
        Catch ex As Exception
            MsgBox(ex.Message)
        Finally
            If Cn IsNot Nothing Then Cn.Close()
        End Try
    End Function

    Private Function CreateCommand(ByVal CommandText As String, ByVal Parameters(,) As String, ByVal Cn As DbConnection) As DbCommand
        ' إنشاء امر
        Dim Command = Fac.CreateCommand
        Command.Connection = Cn
        Command.CommandText = CommandText
        ' استنتاج نوع الأمر
        Dim X = CommandText.ToLower
        If X.StartsWith("update ") OrElse X.StartsWith("insert ") OrElse
                   X.StartsWith("delete ") OrElse X.StartsWith("create ") OrElse
                   X.StartsWith("alter ") OrElse X.StartsWith("drop ") OrElse X.StartsWith("select ") Then
            Command.CommandType = CommandType.Text
        Else
            Command.CommandType = CommandType.StoredProcedure
        End If

        ' إضافة المعاملات
        If Not Parameters Is Nothing Then
            For C = 0 To Parameters.GetLength(0) - 1
                If SqlInjection(Parameters(C, 1)) Then Throw New Exception("هذه القيمة تحتوي على نصوص تشبه الاستعلامات المدسوسة ")
                Dim P = Fac.CreateParameter
                P.ParameterName = Parameters(C, 0)
                P.Value = Parameters(C, 1)
                Command.Parameters.Add(P)
            Next
        End If

        Return Command
    End Function

    Private Function SqlInjection(ByVal Txt As String) As Boolean
        ' سنفحص النص لمعرفة إن كان يحتوي على أية نصوص مشبوهة تشبه الاستعلامات المدسوسة
        Txt = Txt.ToLower
        If Txt.Contains(";") Then Return True
        If Txt.Contains("'") Then Return True
        If Txt.Contains("--") Then Return True
        If Txt.Contains("/*") Then Return True
        If Txt.Contains("*/") Then Return True
        If Txt.Contains("drop") Then Return True
        If Txt.Contains("delete") Then Return True
        If Txt.Contains("update") Then Return True
        If Txt.Contains("insert") Then Return True
        If Txt.Contains("select") Then Return True
        If Txt.Contains("alter") Then Return True
        If Txt.Contains("create") Then Return True
        If Txt.Contains("xp_") Then Return True
        If Txt.Contains("execute") Then Return True
        If Txt.Contains("exec") Then Return True
        If Txt.Contains("sp_executesql") Then Return True
        If Txt.Contains(" aux ") Then Return True
        If Txt.Contains("clock$") Then Return True
        If Txt.Contains(" con ") Then Return True
        If Txt.Contains("config$") Then Return True
        If Txt.Contains(" nul ") Then Return True
        If Txt.Contains(" prn ") Then Return True
        If Txt.Contains("com1") Then Return True
        If Txt.Contains("com2") Then Return True
        If Txt.Contains("com3") Then Return True
        If Txt.Contains("com4") Then Return True
        If Txt.Contains("com5") Then Return True
        If Txt.Contains("com6") Then Return True
        If Txt.Contains("com7") Then Return True
        If Txt.Contains("com8") Then Return True
        If Txt.Contains("lpt1") Then Return True
        If Txt.Contains("lpt2") Then Return True
        If Txt.Contains("lpt3") Then Return True
        If Txt.Contains("lpt4") Then Return True
        If Txt.Contains("lpt5") Then Return True
        If Txt.Contains("lpt6") Then Return True
        If Txt.Contains("lpt7") Then Return True
        If Txt.Contains("lpt8") Then Return True

        Return False
    End Function
End Class
